"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ensureFileSync = exports.ensureFile = void 0;
// Copyright 2018-2024 the Deno authors. All rights reserved. MIT license.
const dntShim = __importStar(require("../../../../../_dnt.shims.js"));
const dirname_js_1 = require("../../path/0.225.1/dirname.js");
const ensure_dir_js_1 = require("./ensure_dir.js");
const _get_file_info_type_js_1 = require("./_get_file_info_type.js");
const _to_path_string_js_1 = require("./_to_path_string.js");
/**
 * Asynchronously ensures that the file exists. If the file that is requested to
 * be created is in directories that do not exist, these directories are created.
 * If the file already exists, it is not modified.
 *
 * Requires the `--allow-read` and `--allow-write` flag.
 *
 * @param filePath The path of the file to ensure, as a string or URL.
 * @returns A void promise that resolves once the file exists.
 *
 * @example
 * ```ts
 * import { ensureFile } from "@std/fs/ensure-file";
 *
 * await ensureFile("./folder/targetFile.dat");
 * ```
 */
async function ensureFile(filePath) {
    try {
        // if file exists
        const stat = await dntShim.Deno.lstat(filePath);
        if (!stat.isFile) {
            throw new Error(`Ensure path exists, expected 'file', got '${(0, _get_file_info_type_js_1.getFileInfoType)(stat)}'`);
        }
    }
    catch (err) {
        // if file not exists
        if (err instanceof dntShim.Deno.errors.NotFound) {
            // ensure dir exists
            await (0, ensure_dir_js_1.ensureDir)((0, dirname_js_1.dirname)((0, _to_path_string_js_1.toPathString)(filePath)));
            // create file
            await dntShim.Deno.writeFile(filePath, new Uint8Array());
            return;
        }
        throw err;
    }
}
exports.ensureFile = ensureFile;
/**
 * Synchronously ensures that the file exists. If the file that is requested to
 * be created is in directories that do not exist, these directories are created.
 * If the file already exists, it is not modified.
 *
 * Requires the `--allow-read` and `--allow-write` flag.
 *
 * @param filePath The path of the file to ensure, as a string or URL.
 * @returns A void value that returns once the file exists.
 *
 * @example
 * ```ts
 * import { ensureFileSync } from "@std/fs/ensure-file";
 *
 * ensureFileSync("./folder/targetFile.dat");
 * ```
 */
function ensureFileSync(filePath) {
    try {
        // if file exists
        const stat = dntShim.Deno.lstatSync(filePath);
        if (!stat.isFile) {
            throw new Error(`Ensure path exists, expected 'file', got '${(0, _get_file_info_type_js_1.getFileInfoType)(stat)}'`);
        }
    }
    catch (err) {
        // if file not exists
        if (err instanceof dntShim.Deno.errors.NotFound) {
            // ensure dir exists
            (0, ensure_dir_js_1.ensureDirSync)((0, dirname_js_1.dirname)((0, _to_path_string_js_1.toPathString)(filePath)));
            // create file
            dntShim.Deno.writeFileSync(filePath, new Uint8Array());
            return;
        }
        throw err;
    }
}
exports.ensureFileSync = ensureFileSync;
