"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.moveSync = exports.move = exports.SubdirectoryMoveError = void 0;
// Copyright 2018-2024 the Deno authors. All rights reserved. MIT license.
const dntShim = __importStar(require("../../../../../_dnt.shims.js"));
const _is_subdir_js_1 = require("./_is_subdir.js");
const _is_same_path_js_1 = require("./_is_same_path.js");
const EXISTS_ERROR = new dntShim.Deno.errors.AlreadyExists("dest already exists.");
/**
 * Error thrown in {@linkcode move} or {@linkcode moveSync} when the
 * destination is a subdirectory of the source.
 */
class SubdirectoryMoveError extends Error {
    /** Constructs a new instance. */
    constructor(src, dest) {
        super(`Cannot move '${src}' to a subdirectory of itself, '${dest}'.`);
        this.name = this.constructor.name;
    }
}
exports.SubdirectoryMoveError = SubdirectoryMoveError;
/**
 * Asynchronously moves a file or directory.
 *
 * @param src The source file or directory as a string or URL.
 * @param dest The destination file or directory as a string or URL.
 * @param options Options for the move operation.
 * @returns A void promise that resolves once the operation completes.
 *
 * @example Basic usage
 * ```ts
 * import { move } from "@std/fs/move";
 *
 * await move("./foo", "./bar");
 * ```
 *
 * This will move the file or directory at `./foo` to `./bar` without
 * overwriting.
 *
 * @example Overwriting
 * ```ts
 * import { move } from "@std/fs/move";
 *
 * await move("./foo", "./bar", { overwrite: true });
 * ```
 *
 * This will move the file or directory at `./foo` to `./bar`, overwriting
 * `./bar` if it already exists.
 */
async function move(src, dest, { overwrite = false } = {}) {
    const srcStat = await dntShim.Deno.stat(src);
    if (srcStat.isDirectory &&
        ((0, _is_subdir_js_1.isSubdir)(src, dest) || (0, _is_same_path_js_1.isSamePath)(src, dest))) {
        throw new SubdirectoryMoveError(src, dest);
    }
    if (overwrite) {
        if ((0, _is_same_path_js_1.isSamePath)(src, dest))
            return;
        try {
            await dntShim.Deno.remove(dest, { recursive: true });
        }
        catch (error) {
            if (!(error instanceof dntShim.Deno.errors.NotFound)) {
                throw error;
            }
        }
    }
    else {
        try {
            await dntShim.Deno.lstat(dest);
            return Promise.reject(EXISTS_ERROR);
        }
        catch {
            // Do nothing...
        }
    }
    await dntShim.Deno.rename(src, dest);
}
exports.move = move;
/**
 * Synchronously moves a file or directory.
 *
 * @param src The source file or directory as a string or URL.
 * @param dest The destination file or directory as a string or URL.
 * @param options Options for the move operation.
 * @returns A void value that returns once the operation completes.
 *
 * @example Basic usage
 * ```ts
 * import { moveSync } from "@std/fs/move";
 *
 * moveSync("./foo", "./bar");
 * ```
 *
 * This will move the file or directory at `./foo` to `./bar` without
 * overwriting.
 *
 * @example Overwriting
 * ```ts
 * import { moveSync } from "@std/fs/move";
 *
 * moveSync("./foo", "./bar", { overwrite: true });
 * ```
 *
 * This will move the file or directory at `./foo` to `./bar`, overwriting
 * `./bar` if it already exists.
 */
function moveSync(src, dest, { overwrite = false } = {}) {
    const srcStat = dntShim.Deno.statSync(src);
    if (srcStat.isDirectory &&
        ((0, _is_subdir_js_1.isSubdir)(src, dest) || (0, _is_same_path_js_1.isSamePath)(src, dest))) {
        throw new SubdirectoryMoveError(src, dest);
    }
    if (overwrite) {
        if ((0, _is_same_path_js_1.isSamePath)(src, dest))
            return;
        try {
            dntShim.Deno.removeSync(dest, { recursive: true });
        }
        catch (error) {
            if (!(error instanceof dntShim.Deno.errors.NotFound)) {
                throw error;
            }
        }
    }
    else {
        try {
            dntShim.Deno.lstatSync(dest);
            throw EXISTS_ERROR;
        }
        catch (error) {
            if (error === EXISTS_ERROR) {
                throw error;
            }
        }
    }
    dntShim.Deno.renameSync(src, dest);
}
exports.moveSync = moveSync;
